@@ -115,7 +115,7 @@ alsa_driver_check_card_type (alsa_driver_t *driver)
 		char tmp[5];
 		strncpy(tmp,strstr(driver->alsa_name_playback,"hw"),4);
 		tmp[4]='\0';
-		printf("control device %s\n",tmp);
+		jack_info("control device %s",tmp);
 		ctl_name = strdup(tmp);
 	} else {
 		ctl_name = strdup(driver->alsa_name_playback);
@@ -233,128 +233,101 @@ alsa_driver_hw_specific (alsa_driver_t *driver, int hw_monitoring,
 static void
 alsa_driver_setup_io_function_pointers (alsa_driver_t *driver)
 {
-	switch (driver->playback_sample_bytes) {
-	case 2:
-		if (driver->playback_interleaved) {
-			driver->channel_copy = memcpy_interleave_d16_s16;
-		} else {
-			driver->channel_copy = memcpy_fake;
-		}
-
-		switch (driver->dither) {
-			case Rectangular:
-			printf("Rectangular dithering at 16 bits\n");
-			driver->write_via_copy = driver->quirk_bswap?
-				sample_move_dither_rect_d16_sSs:
-				sample_move_dither_rect_d16_sS;
-			break;
-
-			case Triangular:
-			printf("Triangular dithering at 16 bits\n");
-			driver->write_via_copy = driver->quirk_bswap?
-				sample_move_dither_tri_d16_sSs:
-				sample_move_dither_tri_d16_sS;
-			break;
-
-			case Shaped:
-			printf("Noise-shaped dithering at 16 bits\n");
-			driver->write_via_copy = driver->quirk_bswap?
-				sample_move_dither_shaped_d16_sSs:
-				sample_move_dither_shaped_d16_sS;
-			break;
-
-			default:
-			driver->write_via_copy = driver->quirk_bswap?
-				sample_move_d16_sSs : sample_move_d16_sS;
-			break;
-		}
-		break;
-
-	case 3:
-		if (driver->playback_interleaved) {
-			driver->channel_copy = memcpy_interleave_d24_s24;
-		} else {
-			driver->channel_copy = memcpy_fake;
-		}
-
-		switch (driver->dither) {
-			case Rectangular:
-			printf("Rectangular dithering at 16 bits\n");
-			driver->write_via_copy = driver->quirk_bswap?
-				sample_move_dither_rect_d24_sSs:
-				sample_move_dither_rect_d24_sS;
-			break;
-
-			case Triangular:
-			printf("Triangular dithering at 16 bits\n");
-			driver->write_via_copy = driver->quirk_bswap?
-				sample_move_dither_tri_d24_sSs:
-				sample_move_dither_tri_d24_sS;
-			break;
-
-			case Shaped:
-			printf("Noise-shaped dithering at 16 bits\n");
-			driver->write_via_copy = driver->quirk_bswap?
-				sample_move_dither_shaped_d24_sSs:
-				sample_move_dither_shaped_d24_sS;
-			break;
-
-			default:
-			driver->write_via_copy = driver->quirk_bswap?
-				sample_move_d24_sSs : sample_move_d24_sS;
-			break;
-		}
-		break;
-
-	case 4:
+	if (SND_PCM_FORMAT_FLOAT_LE == driver->playback_sample_format) {
 		if (driver->playback_interleaved) {
 			driver->channel_copy = memcpy_interleave_d32_s32;
 		} else {
 			driver->channel_copy = memcpy_fake;
 		}
-		
-		switch (driver->dither) {
+		driver->read_via_copy = sample_move_floatLE_sSs;
+		driver->write_via_copy = sample_move_dS_floatLE;
+	} else {
+
+		switch (driver->playback_sample_bytes) {
+		case 2:
+			if (driver->playback_interleaved) {
+				driver->channel_copy = memcpy_interleave_d16_s16;
+			} else {
+				driver->channel_copy = memcpy_fake;
+			}
+			
+			switch (driver->dither) {
 			case Rectangular:
-			printf("Rectangular dithering at 16 bits\n");
-			driver->write_via_copy = driver->quirk_bswap?
-				sample_move_dither_rect_d32u24_sSs:
-				sample_move_dither_rect_d32u24_sS;
-			break;
-
+				jack_info("Rectangular dithering at 16 bits");
+				driver->write_via_copy = driver->quirk_bswap?
+                                        sample_move_dither_rect_d16_sSs:
+                                        sample_move_dither_rect_d16_sS;
+				break;
+				
 			case Triangular:
-			printf("Triangular dithering at 16 bits\n");
-			driver->write_via_copy = driver->quirk_bswap?
-				sample_move_dither_tri_d32u24_sSs:
-				sample_move_dither_tri_d32u24_sS;
-			break;
-
+				jack_info("Triangular dithering at 16 bits");
+				driver->write_via_copy = driver->quirk_bswap?
+                                        sample_move_dither_tri_d16_sSs:
+                                        sample_move_dither_tri_d16_sS;
+				break;
+				
 			case Shaped:
-			printf("Noise-shaped dithering at 16 bits\n");
-			driver->write_via_copy = driver->quirk_bswap?
-				sample_move_dither_shaped_d32u24_sSs:
-				sample_move_dither_shaped_d32u24_sS;
-			break;
-
+				jack_info("Noise-shaped dithering at 16 bits");
+				driver->write_via_copy = driver->quirk_bswap?
+                                        sample_move_dither_shaped_d16_sSs:
+                                        sample_move_dither_shaped_d16_sS;
+				break;
+				
 			default:
-			driver->write_via_copy = driver->quirk_bswap?
-				sample_move_d32u24_sSs : sample_move_d32u24_sS;
+				driver->write_via_copy = driver->quirk_bswap?
+                                        sample_move_d16_sSs : 
+        				sample_move_d16_sS;
+				break;
+			}
 			break;
-		}
-		break;
-	}
+			
+		case 3: /* NO DITHER */
+			if (driver->playback_interleaved) {
+				driver->channel_copy = memcpy_interleave_d24_s24;
+			} else {
+				driver->channel_copy = memcpy_fake;
+			}
+			
+			driver->write_via_copy = driver->quirk_bswap?
+				sample_move_d24_sSs: 
+				sample_move_d24_sS;
 
+			break;
+									
+	 	case 4: /* NO DITHER */
+			if (driver->playback_interleaved) {
+				driver->channel_copy = memcpy_interleave_d32_s32;
+			} else {
+				driver->channel_copy = memcpy_fake;
+			}
+
+			driver->write_via_copy = driver->quirk_bswap?
+				sample_move_d32u24_sSs: 
+				sample_move_d32u24_sS;
+		    break;
+
+		default:
+			jack_error ("impossible sample width (%d) discovered!",
+				    driver->playback_sample_bytes);
+			exit (1);
+		}
+	}
+	
 	switch (driver->capture_sample_bytes) {
 	case 2:
 		driver->read_via_copy = driver->quirk_bswap?
-			sample_move_dS_s16s : sample_move_dS_s16;
+			sample_move_dS_s16s: 
+		        sample_move_dS_s16;
 		break;
 	case 3:
 		driver->read_via_copy = driver->quirk_bswap?
-			sample_move_dS_s24s : sample_move_dS_s24;
+			sample_move_dS_s24s: 
+		        sample_move_dS_s24;
 		break;
 	case 4:
 		driver->read_via_copy = driver->quirk_bswap?
-			sample_move_dS_s32u24s : sample_move_dS_s32u24;
+		 	sample_move_dS_s32u24s: 
+		        sample_move_dS_s32u24;
 		break;
 	}
 }
@@ -377,15 +350,16 @@ alsa_driver_configure_stream (alsa_driver_t *driver, char *device_name,
 		snd_pcm_format_t format;
 		int swapped;
 	} formats[] = {
-		{"32bit little-endian", SND_PCM_FORMAT_S32_LE, IS_LE},
-		{"32bit big-endian", SND_PCM_FORMAT_S32_BE, IS_BE},
+ 	    {"32bit float little-endian", SND_PCM_FORMAT_FLOAT_LE},
+		{"32bit integer little-endian", SND_PCM_FORMAT_S32_LE, IS_LE},
+		{"32bit integer big-endian", SND_PCM_FORMAT_S32_BE, IS_BE},
 		{"24bit little-endian", SND_PCM_FORMAT_S24_3LE, IS_LE},
 		{"24bit big-endian", SND_PCM_FORMAT_S24_3BE, IS_BE},
 		{"16bit little-endian", SND_PCM_FORMAT_S16_LE, IS_LE},
 		{"16bit big-endian", SND_PCM_FORMAT_S16_BE, IS_BE},
 	};
 #define NUMFORMATS (sizeof(formats)/sizeof(formats[0]))
-#define FIRST_16BIT_FORMAT 4
+#define FIRST_16BIT_FORMAT 5
 
 	if ((err = snd_pcm_hw_params_any (handle, hw_params)) < 0)  {
 		jack_error ("ALSA: no playback configurations available (%s)",
@@ -436,7 +410,7 @@ alsa_driver_configure_stream (alsa_driver_t *driver, char *device_name,
 			} else {
 				driver->quirk_bswap = 0;
 			}
-			jack_error ("ALSA: final selected sample format for %s: %s", stream_name, formats[format].Name);
+			jack_info ("ALSA: final selected sample format for %s: %s", stream_name, formats[format].Name);
 			break;
 		}
 	} 
@@ -514,13 +488,14 @@ alsa_driver_configure_stream (alsa_driver_t *driver, char *device_name,
 			    stream_name);
 		return -1;
 	}
-	jack_error ("ALSA: use %d periods for %s", *nperiodsp, stream_name);
-	
+	jack_info ("ALSA: use %d periods for %s", *nperiodsp, stream_name);
+#if 0	
 	if (!jack_power_of_two(driver->frames_per_cycle)) {
 		jack_error("JACK: frames must be a power of two "
 			   "(64, 512, 1024, ...)\n");
 		return -1;
 	}
+#endif
 
 	if ((err = snd_pcm_hw_params_set_buffer_size (handle, hw_params,
 						      *nperiodsp *
@@ -567,7 +542,7 @@ alsa_driver_configure_stream (alsa_driver_t *driver, char *device_name,
 	}
 
 #if 0
-	fprintf (stderr, "set silence size to %lu * %lu = %lu\n",
+	jack_info ("set silence size to %lu * %lu = %lu",
 		 driver->frames_per_cycle, *nperiodsp,
 		 driver->frames_per_cycle * *nperiodsp);
 
@@ -621,8 +596,8 @@ alsa_driver_set_parameters (alsa_driver_t *driver,
 	driver->frames_per_cycle = frames_per_cycle;
 	driver->user_nperiods = user_nperiods;
 
-	fprintf (stderr, "configuring for %" PRIu32 "Hz, period = %"
-		 PRIu32 " frames (%.1f ms), buffer = %" PRIu32 " periods\n",
+	jack_info ("configuring for %" PRIu32 "Hz, period = %"
+		 PRIu32 " frames (%.1f ms), buffer = %" PRIu32 " periods",
 		 rate, frames_per_cycle, (((float)frames_per_cycle / (float) rate) * 1000.0f), user_nperiods);
 	
 	if (driver->capture_handle) {
@@ -759,6 +734,7 @@ alsa_driver_set_parameters (alsa_driver_t *driver,
 
 	if (driver->playback_handle) {
 		switch (driver->playback_sample_format) {
+        case SND_PCM_FORMAT_FLOAT_LE:
 		case SND_PCM_FORMAT_S32_LE:
 		case SND_PCM_FORMAT_S24_3LE:
 		case SND_PCM_FORMAT_S24_3BE:
@@ -776,6 +752,7 @@ alsa_driver_set_parameters (alsa_driver_t *driver,
 
 	if (driver->capture_handle) {
 		switch (driver->capture_sample_format) {
+        case SND_PCM_FORMAT_FLOAT_LE:
 		case SND_PCM_FORMAT_S32_LE:
 		case SND_PCM_FORMAT_S24_3LE:
 		case SND_PCM_FORMAT_S24_3BE:
@@ -1175,7 +1152,7 @@ alsa_driver_xrun_recovery (alsa_driver_t *driver, float *delayed_usecs)
 	    && driver->process_count > XRUN_REPORT_DELAY) {
 		struct timeval now, diff, tstamp;
 		driver->xrun_count++;
-		gettimeofday(&now, 0);
+		snd_pcm_status_get_tstamp(status,&now);
 		snd_pcm_status_get_trigger_tstamp(status, &tstamp);
 		timersub(&now, &tstamp, &diff);
 		*delayed_usecs = diff.tv_sec * 1000000.0 + diff.tv_usec;
@@ -1299,7 +1276,7 @@ alsa_driver_wait (alsa_driver_t *driver, int extra_fd, int *status, float
 		if (poll_result < 0) {
 
 			if (errno == EINTR) {
-				printf ("poll interrupt\n");
+				jack_info ("poll interrupt");
 				// this happens mostly when run
 				// under gdb, or when exiting due to a signal
 				if (under_gdb) {
@@ -1329,8 +1306,8 @@ alsa_driver_wait (alsa_driver_t *driver, int extra_fd, int *status, float
 		}
 
 #ifdef DEBUG_WAKEUP
-		fprintf (stderr, "%" PRIu64 ": checked %d fds, %" PRIu64
-			 " usecs since poll entered\n", poll_ret, nfds,
+		jack_info ("%" PRIu64 ": checked %d fds, %" PRIu64
+			 " usecs since poll entered", poll_ret, nfds,
 			 poll_ret - poll_enter);
 #endif
 
@@ -1368,8 +1345,8 @@ alsa_driver_wait (alsa_driver_t *driver, int extra_fd, int *status, float
 			if (revents & POLLOUT) {
 				need_playback = 0;
 #ifdef DEBUG_WAKEUP
-				fprintf (stderr, "%" PRIu64
-					 " playback stream ready\n",
+				jack_info ("%" PRIu64
+					 " playback stream ready",
 					 poll_ret);
 #endif
 			}
@@ -1391,8 +1368,8 @@ alsa_driver_wait (alsa_driver_t *driver, int extra_fd, int *status, float
 			if (revents & POLLIN) {
 				need_capture = 0;
 #ifdef DEBUG_WAKEUP
-				fprintf (stderr, "%" PRIu64
-					 " capture stream ready\n",
+				jack_info ("%" PRIu64
+					 " capture stream ready",
 					 poll_ret);
 #endif
 			}
@@ -1449,8 +1426,8 @@ alsa_driver_wait (alsa_driver_t *driver, int extra_fd, int *status, float
 	avail = capture_avail < playback_avail ? capture_avail : playback_avail;
 
 #ifdef DEBUG_WAKEUP
-	fprintf (stderr, "wakeup complete, avail = %lu, pavail = %lu "
-		 "cavail = %lu\n",
+	jack_info ("wakeup complete, avail = %lu, pavail = %lu "
+		 "cavail = %lu",
 		 avail, playback_avail, capture_avail);
 #endif
 
@@ -1600,7 +1577,7 @@ alsa_driver_read (alsa_driver_t *driver, jack_nframes_t nframes)
 		if ((err = snd_pcm_mmap_commit (driver->capture_handle,
 				offset, contiguous)) < 0) {
 			jack_error ("ALSA: could not complete read of %"
-				PRIu32 " frames: error = %d\n", contiguous, err);
+				PRIu32 " frames: error = %d", contiguous, err);
 			return -1;
 		}
 
@@ -1938,6 +1915,11 @@ alsa_driver_delete (alsa_driver_t *driver)
 	}
 	jack_slist_free (driver->clock_sync_listeners);
 
+	if (driver->ctl_handle) {
+		snd_ctl_close (driver->ctl_handle);
+		driver->ctl_handle = 0;
+	} 
+
 	if (driver->capture_handle) {
 		snd_pcm_close (driver->capture_handle);
 		driver->capture_handle = 0;
@@ -2011,8 +1993,8 @@ alsa_driver_new (char *name, char *playback_alsa_device,
 
 	alsa_driver_t *driver;
 
-	printf ("creating alsa driver ... %s|%s|%" PRIu32 "|%" PRIu32
-		"|%" PRIu32"|%" PRIu32"|%" PRIu32 "|%s|%s|%s|%s\n",
+	jack_info ("creating alsa driver ... %s|%s|%" PRIu32 "|%" PRIu32
+		"|%" PRIu32"|%" PRIu32"|%" PRIu32 "|%s|%s|%s|%s",
 		playing ? playback_alsa_device : "-",
 		capturing ? capture_alsa_device : "-", 
 		frames_per_cycle, user_nperiods, rate,
@@ -2343,7 +2325,7 @@ dither_opt (char c, DitherAlgorithm* dither)
 		break;
 		
 	default:
-		fprintf (stderr, "ALSA driver: illegal dithering mode %c\n", c);
+		jack_error ("ALSA driver: illegal dithering mode %c", c);
 		return -1;
 	}
 	return 0;
@@ -2600,7 +2582,7 @@ driver_initialize (jack_client_t *client, const JSList * params)
 
 		case 'r':
 		        srate = param->value.ui;
-			fprintf (stderr, "apparent rate = %d\n", srate);
+			jack_info ("apparent rate = %d", srate);
 		        break;
 			
 		case 'p':
