@@ -21,6 +21,7 @@
 #define __jack_alsa_driver_h__
 
 #include <alsa/asoundlib.h>
+#include <alsa/pcm.h>
 #include <jack/bitset.h>
 
 #if __BYTE_ORDER == __LITTLE_ENDIAN
@@ -31,14 +32,12 @@
 #define IS_BE 0
 #endif
 
-
-
 #include <jack/types.h>
 #include <jack/hardware.h>
 #include <jack/driver.h>
-#include <jack/memops.h>
 #include <jack/jack.h>
 
+#include "memops.h"
 #include "../alsa-midi/alsa_midi.h"
 
 typedef void (*ReadCopyFunction)  (jack_default_audio_sample_t *dst, char *src,
