@@ -34,9 +34,26 @@
 #include <sys/types.h>
 #include <sys/time.h>
 
+#ifndef POST_PACKED_STRUCTURE
+#ifdef __GNUC__
+/* POST_PACKED_STRUCTURE needs to be a macro which
+   expands into a compiler directive. The directive must
+   tell the compiler to arrange the preceding structure
+   declaration so that it is packed on byte-boundaries rather 
+   than use the natural alignment of the processor and/or
+   compiler.
+*/
+#define POST_PACKED_STRUCTURE __attribute__((__packed__))
+#else
+/* Add other things here for non-gcc platforms */
+#endif
+#endif
+
 /* Needed by <sysdeps/time.h> */
 extern void jack_error (const char *fmt, ...);
 
+extern void jack_info (const char *fmt, ...);
+
 #include <jack/jack.h>
 #include <jack/types.h>
 #include <jack/port.h>
@@ -59,9 +76,25 @@ const char* jack_clock_source_name (jack_timer_type_t);
 #include <sysdeps/mach_port.h>
 #endif
 
+#include <jack/messagebuffer.h>
+
 #ifdef DEBUG_ENABLED
+
+/* grab thread id instead of PID on linux */
+#if defined(__gnu_linux__)
+    #ifdef gettid  /* glibc has a version */
+        #define GETTID() gettid()
+    #else /* use our own version */
+        #include <sys/syscall.h>
+        #define GETTID() syscall(__NR_gettid)
+    #endif
+#else
+    #define GETTID() getpid()
+#endif
+
 #define DEBUG(format,args...) \
-	fprintf (stderr, "jack:%5d:%" PRIu64 " %s:%s:%d: " format "\n", getpid(), jack_get_microseconds(), __FILE__, __FUNCTION__, __LINE__ , ## args)
+	MESSAGE("jack:%5d:%" PRIu64 " %s:%s:%d: " format "", GETTID(), jack_get_microseconds(), __FILE__, __FUNCTION__, __LINE__ , ## args)
+
 #else
 #if JACK_CPP_VARARGS_BROKEN
     #define DEBUG(format...)
@@ -118,7 +151,7 @@ typedef struct {
 	int32_t  reset_pending;      /* xrun happened, deal with it */
 	float    filter_coefficient; /* set once, never altered */
 
-} jack_frame_timer_t;
+} POST_PACKED_STRUCTURE jack_frame_timer_t;
 
 /* JACK engine shared memory data structure. */
 typedef struct {
@@ -147,6 +180,7 @@ typedef struct {
     int8_t		  do_mlock;
     int8_t		  do_munlock;
     int32_t		  client_priority;
+    int32_t		  max_client_priority;
     int32_t		  has_capabilities;
     float                 cpu_load;
     float		  xrun_delayed_usecs;
@@ -157,7 +191,7 @@ typedef struct {
     jack_port_type_info_t port_types[JACK_MAX_PORT_TYPES];
     jack_port_shared_t    ports[0];
 
-} jack_control_t;
+} POST_PACKED_STRUCTURE jack_control_t;
 
 typedef enum  {
   BufferSizeChange,
@@ -188,7 +222,7 @@ typedef struct {
 	jack_port_type_id_t ptid;
 	jack_port_id_t other_id;
     } y;
-} jack_event_t;
+} POST_PACKED_STRUCTURE jack_event_t;
 
 typedef enum {
 	ClientInternal, /* connect request just names .so */
@@ -227,46 +261,28 @@ typedef volatile struct {
     volatile uint64_t	finished_at;
     volatile int32_t	last_status;         /* w: client, r: engine and client */
 
-    /* JOQ: all these pointers are trouble for 32/64 compatibility,
-     * they should move to non-shared memory. 
+    /* indicators for whether callbacks have been set for this client.
+       We do not include ptrs to the callbacks here (or their arguments)
+       so that we can avoid 32/64 bit pointer size mismatches between
+       the jack server and a client. The pointers are in the client-
+       local structure which is part of the libjack compiled for
+       either 32 bit or 64 bit clients.
      */
+    volatile uint8_t	process_cbset;
+    volatile uint8_t	thread_init_cbset;
+    volatile uint8_t	bufsize_cbset;
+    volatile uint8_t	srate_cbset;
+    volatile uint8_t	port_register_cbset;
+    volatile uint8_t	port_connect_cbset;
+    volatile uint8_t	graph_order_cbset;
+    volatile uint8_t	xrun_cbset;
+    volatile uint8_t	sync_cb_cbset;
+    volatile uint8_t	timebase_cb_cbset;
+    volatile uint8_t	freewheel_cb_cbset;
+    volatile uint8_t	client_register_cbset;
+    volatile uint8_t	thread_cb_cbset;
 
-    /* callbacks 
-     */
-    JackProcessCallback process;
-    void *process_arg;
-    JackThreadInitCallback thread_init;
-    void *thread_init_arg;
-    JackBufferSizeCallback bufsize;
-    void *bufsize_arg;
-    JackSampleRateCallback srate;
-    void *srate_arg;
-    JackPortRegistrationCallback port_register;
-    void *port_register_arg;
-    JackPortConnectCallback port_connect;
-    void *port_connect_arg;
-    JackGraphOrderCallback graph_order;
-    void *graph_order_arg;
-    JackXRunCallback xrun;
-    void *xrun_arg;
-    JackSyncCallback sync_cb;
-    void *sync_arg;
-    JackTimebaseCallback timebase_cb;
-    void *timebase_arg;
-    JackFreewheelCallback freewheel_cb;
-    void *freewheel_arg;
-    JackClientRegistrationCallback client_register;	
-    void *client_register_arg;
-
-    /* external clients: set by libjack
-     * internal clients: set by engine */
-    int (*deliver_request)(void*, jack_request_t*); /* JOQ: 64/32 bug! */
-    void *deliver_arg;
-
-    /* for engine use only */
-    void *private_client;
-
-} jack_client_control_t;
+} POST_PACKED_STRUCTURE jack_client_control_t;
 
 typedef struct {
     
@@ -279,14 +295,14 @@ typedef struct {
     char object_path[PATH_MAX+1];
     char object_data[1024];
 
-} jack_client_connect_request_t;
+} POST_PACKED_STRUCTURE jack_client_connect_request_t;
 
 typedef struct {
 
     jack_status_t status;
 
-    jack_shm_info_t client_shm;
-    jack_shm_info_t engine_shm;
+    jack_shm_registry_index_t client_shm_index;
+    jack_shm_registry_index_t engine_shm_index;
 
     char	fifo_prefix[PATH_MAX+1];
 
@@ -295,24 +311,29 @@ typedef struct {
 
     char name[JACK_CLIENT_NAME_SIZE];	/* unique name, if assigned */
 
-    /* these two are valid only for internal clients */
-    jack_client_control_t *client_control;
-    jack_control_t        *engine_control;
+    /* these are actually pointers, but they must
+       be the same size regardless of whether the
+       server and/or client are 64 bit or 32 bit.
+       force them to be 64 bit.
+    */
+
+    uint64_t client_control;
+    uint64_t engine_control;
 
 #ifdef JACK_USE_MACH_THREADS
     /* specific resources for server/client real-time thread communication */
     int32_t	portnum;
 #endif
 
-} jack_client_connect_result_t;
+} POST_PACKED_STRUCTURE jack_client_connect_result_t;
 
 typedef struct {
     jack_client_id_t client_id;
-} jack_client_connect_ack_request_t;
+} POST_PACKED_STRUCTURE jack_client_connect_ack_request_t;
 
 typedef struct {
     int8_t status;
-} jack_client_connect_ack_result_t;
+} POST_PACKED_STRUCTURE jack_client_connect_ack_result_t;
 
 typedef enum {
 	RegisterPort = 1,
@@ -343,7 +364,8 @@ typedef enum {
 
 struct _jack_request {
     
-    RequestType type;
+    //RequestType type;
+    uint32_t type;
     union {
 	struct {
 	    char name[JACK_PORT_NAME_SIZE];
@@ -352,33 +374,43 @@ struct _jack_request {
 	    jack_shmsize_t   buffer_size;
 	    jack_port_id_t   port_id;
 	    jack_client_id_t client_id;
-	} port_info;
+	} POST_PACKED_STRUCTURE port_info;
 	struct {
 	    char source_port[JACK_PORT_NAME_SIZE];
 	    char destination_port[JACK_PORT_NAME_SIZE];
-	} connect;
+	} POST_PACKED_STRUCTURE connect;
 	struct {
 	    int32_t nports;
-	    const char **ports;		/* JOQ: 32/64 problem? */
-	} port_connections;
+	    const char **ports;	/* this is only exposed to internal clients, so there
+				   is no 64/32 issue. external clients read the ports
+				   one by one from the server, and allocate their
+				   own "ports" array in their own address space.
+
+				   we are lucky, because this is part of a union
+				   whose other components are bigger than this one.
+				   otherwise it would change structure size when
+				   comparing the 64 and 32 bit versions.
+				*/
+	} POST_PACKED_STRUCTURE port_connections;
 	struct {
 	    jack_client_id_t client_id;
 	    int32_t conditional;
-	} timebase;
+	} POST_PACKED_STRUCTURE timebase;
 	struct {
-	    jack_options_t options;
+	    //jack_options_t options;
+	    uint32_t options;
 	    jack_client_id_t id;
 	    char name[JACK_CLIENT_NAME_SIZE];
 	    char path[PATH_MAX+1];
 	    char init[JACK_LOAD_INIT_LIMIT];
-	} intclient;
+	} POST_PACKED_STRUCTURE intclient;
 	jack_client_id_t client_id;
 	jack_nframes_t nframes;
 	jack_time_t timeout;
         pid_t cap_pid;
-    } x;
+    } POST_PACKED_STRUCTURE x;
     int32_t status;
-};
+} POST_PACKED_STRUCTURE;
 
 /* Per-client structure allocated in the server's address space.
  * It's here because its not part of the engine structure.
@@ -412,7 +444,12 @@ typedef struct _jack_client_internal {
     int running;
     int portnum;
 #endif /* JACK_USE_MACH_THREADS */
-    
+   
+    /* external clients: set by libjack
+     * internal clients: set by engine */
+    //int (*deliver_request)(void*, jack_request_t*); /* JOQ: 64/32 bug! */
+    //void *deliver_arg;
+    jack_client_t *private_client;
 } jack_client_internal_t;
 
 typedef struct _jack_thread_arg {
