@@ -25,6 +25,7 @@
 
 #include <errno.h>
 #include <stdio.h>
+#include <unistd.h>
 #include <string.h>
 
 #include <jack/internal.h>
@@ -37,7 +38,7 @@
 #include "clientengine.h"
 #include "transengine.h"
 
-#define JACK_ERROR_WITH_SOCKETS 10000000
+#include "libjack/local.h"
 
 static void
 jack_client_disconnect_ports (jack_engine_t *engine,
@@ -68,7 +69,10 @@ jack_client_do_deactivate (jack_engine_t *engine,
 			   jack_client_internal_t *client, int sort_graph)
 {
 	/* caller must hold engine->client_lock and must have checked for and/or
-	 *   cleared all connections held by client. */
+	 *   cleared all connections held by client. 
+	 */
+	VERBOSE(engine,"+++ deactivate %s", client->control->name);
+
 	client->control->active = FALSE;
 
 	jack_transport_client_exit (engine, client);
@@ -77,7 +81,7 @@ jack_client_do_deactivate (jack_engine_t *engine,
 	    engine->external_client_cnt > 0) {	
 		engine->external_client_cnt--;
 	}
-
+	
 	if (sort_graph) {
 		jack_sort_graph (engine);
 	}
@@ -87,12 +91,12 @@ jack_client_do_deactivate (jack_engine_t *engine,
 static void
 jack_zombify_client (jack_engine_t *engine, jack_client_internal_t *client)
 {
-	VERBOSE (engine, "removing client \"%s\" from the processing chain\n",
+	VERBOSE (engine, "removing client \"%s\" from the processing chain",
 		 client->control->name);
 
 	/* caller must hold the client_lock */
 
-	/* this stops jack_deliver_event() from doing anything */
+	/* this stops jack_deliver_event() from contacing this client */
 
 	client->control->dead = TRUE;
 
@@ -103,13 +107,11 @@ jack_zombify_client (jack_engine_t *engine, jack_client_internal_t *client)
 static void
 jack_remove_client (jack_engine_t *engine, jack_client_internal_t *client)
 {
-	/* called *without* the request_lock */
-	unsigned int i;
 	JSList *node;
 
-	/* caller must hold the client_lock */
+	/* caller must write-hold the client lock */
 
-	VERBOSE (engine, "removing client \"%s\"\n", client->control->name);
+	VERBOSE (engine, "removing client \"%s\"", client->control->name);
 
 	/* if its not already a zombie, make it so */
 
@@ -123,22 +125,6 @@ jack_remove_client (jack_engine_t *engine, jack_client_internal_t *client)
 	
 		close (client->event_fd);
 		close (client->request_fd);
-
-		/* rearrange the pollfd array so that things work right the 
-		   next time we go into poll(2).
-		*/
-		
-		for (i = 0; i < engine->pfd_max; i++) {
-			if (engine->pfd[i].fd == client->request_fd) {
-				if (i+1 < engine->pfd_max) {
-					memmove (&engine->pfd[i],
-						 &engine->pfd[i+1],
-						 sizeof (struct pollfd)
-						 * (engine->pfd_max - i));
-				}
-				engine->pfd_max--;
-			}
-		}
 	}
 
 	for (node = engine->clients; node; node = jack_slist_next (node)) {
@@ -154,10 +140,68 @@ jack_remove_client (jack_engine_t *engine, jack_client_internal_t *client)
 	jack_client_delete (engine, client);
 
 	/* ignore the driver, which counts as a client. */
+
 	if (engine->temporary && (jack_slist_length(engine->clients) <= 1)) {
-		exit(0);
+		exit (0);
+	}
+}
+
+static void
+jack_wake_server_thread (jack_engine_t* engine)
+{
+	char c = 0;
+	/* we don't actually care if this fails */
+	VERBOSE (engine, "waking server thread");
+	write (engine->cleanup_fifo[1], &c, 1);
+}
+
+void
+jack_check_clients (jack_engine_t* engine, int with_timeout_check)
+{
+	/* CALLER MUST HOLD graph read lock */
+
+	JSList* node;
+	jack_client_internal_t* client;
+	int errs = 0;
+
+	for (node = engine->clients; node; node = jack_slist_next (node)) {
+
+		client = (jack_client_internal_t *) node->data;
+
+		if (client->error) {
+			errs++;
+			continue;
+		}
+
+		if (with_timeout_check) {
+
+			/* we can only consider the timeout a client error if
+			 * it actually woke up.  its possible that the kernel
+			 * scheduler screwed us up and never woke up the
+			 * client in time. sigh.
+			 */
+			
+			VERBOSE (engine, "checking client %s: awake at %" PRIu64 " finished at %" PRIu64, 
+				 client->control->name,
+				 client->control->awake_at,
+				 client->control->finished_at);
+			
+			if (client->control->awake_at > 0) {
+				if (client->control->finished_at == 0) {
+					client->control->timed_out++;
+					client->error++;
+					VERBOSE (engine, "client %s has timed out", client->control->name);
+				}
+			}
+		}
+	}
+		
+	if (errs) {
+		jack_lock_problems (engine);
+		engine->problems++;
+		jack_unlock_problems (engine);
+		jack_wake_server_thread (engine);
 	}
-	
 }
 
 void
@@ -166,7 +210,11 @@ jack_remove_clients (jack_engine_t* engine)
 	JSList *tmp, *node;
 	int need_sort = FALSE;
 	jack_client_internal_t *client;
-	
+
+	/* CALLER MUST HOLD GRAPH LOCK */
+
+	VERBOSE (engine, "++ Removing failed clients ...");
+
 	/* remove all dead clients */
 
 	for (node = engine->clients; node; ) {
@@ -174,6 +222,8 @@ jack_remove_clients (jack_engine_t* engine)
 		tmp = jack_slist_next (node);
 		
 		client = (jack_client_internal_t *) node->data;
+
+		VERBOSE(engine, "client %s error status %d", client->control->name, client->error);
 		
 		if (client->error) {
 			
@@ -187,7 +237,7 @@ jack_remove_clients (jack_engine_t* engine)
 			if (client->error >= JACK_ERROR_WITH_SOCKETS) {
 				VERBOSE (engine, "removing failed "
 					 "client %s state = %s errors"
-					 " = %d\n", 
+					 " = %d", 
 					 client->control->name,
 					 jack_client_state_name (client),
 					 client->error);
@@ -197,7 +247,7 @@ jack_remove_clients (jack_engine_t* engine)
 			} else {
 				VERBOSE (engine, "client failure: "
 					 "client %s state = %s errors"
-					 " = %d\n", 
+					 " = %d", 
 					 client->control->name,
 					 jack_client_state_name (client),
 					 client->error);
@@ -220,6 +270,8 @@ jack_remove_clients (jack_engine_t* engine)
 	}
 	
 	jack_engine_reset_rolling_usecs (engine);
+
+	VERBOSE (engine, "-- Removing failed clients ...");
 }
 
 static int
@@ -268,7 +320,7 @@ jack_client_unload (jack_client_internal_t *client)
 {
 	if (client->handle) {
 		if (client->finish) {
-			client->finish (client->control->process_arg);
+			client->finish (client->private_client->process_arg);
 		}
 		dlclose (client->handle);
 	}
@@ -280,7 +332,7 @@ jack_client_by_name (jack_engine_t *engine, const char *name)
 	jack_client_internal_t *client = NULL;
 	JSList *node;
 
-	jack_lock_graph (engine);
+	jack_rdlock_graph (engine);
 
 	for (node = engine->clients; node; node = jack_slist_next (node)) {
 		if (strcmp ((const char *) ((jack_client_internal_t *)
@@ -301,7 +353,7 @@ jack_client_id_by_name (jack_engine_t *engine, const char *name)
 	jack_client_id_t id = 0;	/* NULL client ID */
 	JSList *node;
 
-	jack_lock_graph (engine);
+	jack_rdlock_graph (engine);
 
 	for (node = engine->clients; node; node = jack_slist_next (node)) {
 		if (strcmp ((const char *) ((jack_client_internal_t *)
@@ -327,6 +379,7 @@ jack_client_internal_by_id (jack_engine_t *engine, jack_client_id_t id)
 	/* call tree ***MUST HOLD*** the graph lock */
 
 	for (node = engine->clients; node; node = jack_slist_next (node)) {
+
 		if (((jack_client_internal_t *) node->data)->control->id
 		    == id) {
 			client = (jack_client_internal_t *) node->data;
@@ -465,23 +518,38 @@ jack_setup_client_control (jack_engine_t *engine, int fd,
 	client->subgraph_start_fd = -1;
 	client->subgraph_wait_fd = -1;
 
-	client->control->process = NULL;
-	client->control->process_arg = NULL;
-	client->control->bufsize = NULL;
-	client->control->bufsize_arg = NULL;
-	client->control->srate = NULL;
-	client->control->srate_arg = NULL;
-	client->control->xrun = NULL;
-	client->control->xrun_arg = NULL;
-	client->control->port_register = NULL;
-	client->control->port_register_arg = NULL;
-	client->control->port_connect = NULL;
-	client->control->port_connect_arg = NULL;
-	client->control->graph_order = NULL;
-	client->control->graph_order_arg = NULL;
-	client->control->client_register = NULL;
-	client->control->client_register_arg = NULL;
+	client->control->process_cbset = FALSE;
+	client->control->bufsize_cbset = FALSE;
+	client->control->srate_cbset = FALSE;
+	client->control->xrun_cbset = FALSE;
+	client->control->port_register_cbset = FALSE;
+	client->control->port_connect_cbset = FALSE;
+	client->control->graph_order_cbset = FALSE;
+	client->control->client_register_cbset = FALSE;
+	client->control->thread_cb_cbset = FALSE;
 
+#if 0
+	if (type != ClientExternal) {
+	    client->process = NULL;
+	    client->process_arg = NULL;
+	    client->bufsize = NULL;
+	    client->bufsize_arg = NULL;
+	    client->srate = NULL;
+	    client->srate_arg = NULL;
+	    client->xrun = NULL;
+	    client->xrun_arg = NULL;
+	    client->port_register = NULL;
+	    client->port_register_arg = NULL;
+	    client->port_connect = NULL;
+	    client->port_connect_arg = NULL;
+	    client->graph_order = NULL;
+	    client->graph_order_arg = NULL;
+	    client->client_register = NULL;
+	    client->client_register_arg = NULL;
+	    client->thread_cb = NULL;
+	    client->thread_cb_arg = NULL;
+	}
+#endif
 	jack_transport_client_new (client);
         
 #ifdef JACK_USE_MACH_THREADS
@@ -527,18 +595,28 @@ setup_client (jack_engine_t *engine, ClientType type, char *name,
 	}
 
 	VERBOSE (engine, "new client: %s, id = %" PRIu32
-		 " type %d @ %p fd = %d\n", 
+		 " type %d @ %p fd = %d", 
 		 client->control->name, client->control->id, 
 		 type, client->control, client_fd);
 
 	if (jack_client_is_internal(client)) {
 
+	    // XXX: do i need to lock the graph here ?
+	    // i moved this one up in the init process, lets see what happens.
+
+		/* Internal clients need to make regular JACK API
+		 * calls, which need a jack_client_t structure.
+		 * Create one here.
+		 */
+		client->private_client =
+			jack_client_alloc_internal (client->control, engine);
+
 		/* Set up the pointers necessary for the request
 		 * system to work.  The client is in the same address
 		 * space */
 
-		client->control->deliver_request = internal_client_request;
-		client->control->deliver_arg = engine;
+		client->private_client->deliver_request = internal_client_request;
+		client->private_client->deliver_arg = engine;
 	}
 
 	/* add new client to the clients list */
@@ -548,12 +626,6 @@ setup_client (jack_engine_t *engine, ClientType type, char *name,
 	
 	if (jack_client_is_internal(client)) {
 
-		/* Internal clients need to make regular JACK API
-		 * calls, which need a jack_client_t structure.
-		 * Create one here.
-		 */
-		client->control->private_client =
-			jack_client_alloc_internal (client->control, engine);
 
 		jack_unlock_graph (engine);
 
@@ -563,12 +635,12 @@ setup_client (jack_engine_t *engine, ClientType type, char *name,
 		if (client->control->type == ClientInternal) {
 
 			pthread_mutex_unlock (&engine->request_lock);
-			if (client->initialize (client->control->private_client,
+			if (client->initialize (client->private_client,
 						object_data)) {
 
 				/* failed: clean up client data */
 				VERBOSE (engine,
-					 "%s jack_initialize() failed!\n",
+					 "%s jack_initialize() failed!",
 					 client->control->name);
 				jack_lock_graph (engine);
 				jack_remove_client (engine, client);
@@ -582,18 +654,6 @@ setup_client (jack_engine_t *engine, ClientType type, char *name,
 		}
 
 	} else {			/* external client */
-		
-		if (engine->pfd_max >= engine->pfd_size) {
-			engine->pfd = (struct pollfd *)
-				realloc (engine->pfd, sizeof (struct pollfd)
-					 * (engine->pfd_size + 16));
-			engine->pfd_size += 16;
-		}
-		
-		engine->pfd[engine->pfd_max].fd = client->request_fd;
-		engine->pfd[engine->pfd_max].events =
-			POLLIN|POLLPRI|POLLERR|POLLHUP|POLLNVAL;
-		engine->pfd_max++;
 
 		jack_unlock_graph (engine);
 	}
@@ -628,7 +688,7 @@ handle_unload_client (jack_engine_t *engine, jack_client_id_t id)
 	jack_lock_graph (engine);
 
 	if ((client = jack_client_internal_by_id (engine, id))) {
-		VERBOSE (engine, "unloading client \"%s\"\n",
+		VERBOSE (engine, "unloading client \"%s\"",
 			 client->control->name);
 		jack_remove_client (engine, client);
 		status = 0;
@@ -694,8 +754,8 @@ jack_client_create (jack_engine_t *engine, int client_fd)
 		res.status |= JackFailure; /* just making sure */
 		return -1;
 	}
-	res.client_shm = client->control_shm;
-	res.engine_shm = engine->control_shm;
+	res.client_shm_index = client->control_shm.index;
+	res.engine_shm_index = engine->control_shm.index;
 	res.realtime = engine->control->real_time;
 	res.realtime_priority = engine->rtpriority - 1;
 	strncpy (res.name, req.name, sizeof(res.name));
@@ -706,8 +766,16 @@ jack_client_create (jack_engine_t *engine, int client_fd)
 #endif
 	
 	if (jack_client_is_internal(client)) {
-		res.client_control = client->control;
-		res.engine_control = engine->control;
+		/* the ->control pointers are for an internal client
+		   so we know they are the right sized pointers
+		   for this server. however, to keep the result
+		   structure the same size for both 32 and 64 bit
+		   clients/servers, the result structure stores
+		   them as 64 bit integer, so we have to do a slightly
+		   forced cast here.
+		*/
+		res.client_control = (uint64_t) ((intptr_t) client->control);
+		res.engine_control = (uint64_t) ((intptr_t) engine->control);
 	} else {
 		strcpy (res.fifo_prefix, engine->fifo_prefix);
 	}
@@ -800,15 +868,13 @@ jack_client_deactivate (jack_engine_t *engine, jack_client_id_t id)
 }	
 
 int
-jack_client_disconnect (jack_engine_t *engine, int fd)
+jack_mark_client_socket_error (jack_engine_t *engine, int fd)
 {
+	/* CALLER MUST HOLD GRAPH LOCK */
+
 	jack_client_internal_t *client = 0;
 	JSList *node;
 
-#ifndef DEFER_CLIENT_REMOVE_TO_AUDIO_THREAD
-
-        jack_lock_graph (engine);
-
         for (node = engine->clients; node; node = jack_slist_next (node)) {
 
                 if (jack_client_is_internal((jack_client_internal_t *)
@@ -823,40 +889,13 @@ jack_client_disconnect (jack_engine_t *engine, int fd)
         }
 
         if (client) {
-		VERBOSE (engine, "removing disconnected client %s state = "
-			 "%s errors = %d\n", client->control->name,
+		VERBOSE (engine, "marking client %s with SOCKET error state = "
+			 "%s errors = %d", client->control->name,
 			 jack_client_state_name (client),
 			 client->error);
-		jack_remove_client(engine, client);
-		jack_sort_graph (engine);
+		client->error += JACK_ERROR_WITH_SOCKETS;
 	}
 
-        jack_unlock_graph (engine);
-
-#else /* DEFER_CLIENT_REMOVE_TO_AUDIO_THREAD */
-
-	jack_lock_graph (engine);
-
-	for (node = engine->clients; node; node = jack_slist_next (node)) {
-
-		if (jack_client_is_internal((jack_client_internal_t *)
-					    node->data)) {
-			continue;
-		}
-
-		if (((jack_client_internal_t *) node->data)->request_fd == fd) {
-			client = (jack_client_internal_t *) node->data;
-			if (client->error < JACK_ERROR_WITH_SOCKETS) {
-				client->error += JACK_ERROR_WITH_SOCKETS;
-			}
-			break;
-		}
-	}
-
-	jack_unlock_graph (engine);
-
-#endif /* DEFER_CLIENT_REMOVE_TO_AUDIO_THREAD */
-
 	return 0;
 }
 
@@ -868,7 +907,7 @@ jack_client_delete (jack_engine_t *engine, jack_client_internal_t *client)
 	if (jack_client_is_internal (client)) {
 
 		jack_client_unload (client);
-		free (client->control->private_client);
+		free (client->private_client);
 		free ((void *) client->control);
 
 	} else {
@@ -906,7 +945,7 @@ jack_intclient_load_request (jack_engine_t *engine, jack_request_t *req)
 	jack_status_t status = 0;
 
 	VERBOSE (engine, "load internal client %s from %s, init `%s', "
-		 "options: 0x%x\n", req->x.intclient.name,
+		 "options: 0x%x", req->x.intclient.name,
 		 req->x.intclient.path, req->x.intclient.init,
 		 req->x.intclient.options);
 
@@ -917,7 +956,7 @@ jack_intclient_load_request (jack_engine_t *engine, jack_request_t *req)
 	if (client == NULL) {
 		status |= JackFailure;	/* just making sure */
 		req->x.intclient.id = 0;
-		VERBOSE (engine, "load failed, status = 0x%x\n", status);
+		VERBOSE (engine, "load failed, status = 0x%x", status);
 	} else {
 		req->x.intclient.id = client->control->id;
 	}
@@ -930,7 +969,7 @@ jack_intclient_name_request (jack_engine_t *engine, jack_request_t *req)
 {
 	jack_client_internal_t *client;
 
-	jack_lock_graph (engine);
+	jack_rdlock_graph (engine);
 	if ((client = jack_client_internal_by_id (engine,
 						  req->x.intclient.id))) {
 		strncpy ((char *) req->x.intclient.name,
@@ -955,7 +994,8 @@ jack_intclient_unload_request (jack_engine_t *engine, jack_request_t *req)
 			handle_unload_client (engine, req->x.intclient.id);
 		pthread_mutex_lock (&engine->request_lock);
 	} else {
-		VERBOSE (engine, "invalid unload request\n");
+		VERBOSE (engine, "invalid unload request");
 		req->status = JackFailure;
 	}
 }
+
