@@ -148,14 +148,14 @@ jack_main (jack_driver_desc_t * driver_desc, JSList * driver_params)
 				       temporary, verbose, client_timeout,
 				       port_max, getpid(), frame_time_offset, 
 				       nozombies, drivers)) == 0) {
-		fprintf (stderr, "cannot create engine\n");
+		jack_error ("cannot create engine");
 		return -1;
 	}
 
-	fprintf (stderr, "loading driver ..\n");
+	jack_info ("loading driver ..");
 	
 	if (jack_engine_load_driver (engine, driver_desc, driver_params)) {
-		fprintf (stderr, "cannot load driver module %s\n",
+		jack_error ("cannot load driver module %s",
 			 driver_desc->name);
 		goto error;
 	}
@@ -181,7 +181,7 @@ jack_main (jack_driver_desc_t * driver_desc, JSList * driver_params)
 	}
 	
 	if (verbose) {
-		fprintf (stderr, "%d waiting for signals\n", getpid());
+		jack_info ("%d waiting for signals", getpid());
 	}
 
 	waiting = TRUE;
@@ -189,7 +189,7 @@ jack_main (jack_driver_desc_t * driver_desc, JSList * driver_params)
 	while (waiting) {
 		sigwait (&signals, &sig);
 
-		fprintf (stderr, "jack main caught signal %d\n", sig);
+		jack_info ("jack main caught signal %d", sig);
 		
 		switch (sig) {
 		case SIGUSR1:
@@ -241,7 +241,7 @@ jack_drivers_get_descriptor (JSList * drivers, const char * sofile)
 	sprintf (filename, "%s/%s", driver_dir, sofile);
 
 	if (verbose) {
-		fprintf (stderr, "getting driver descriptor from %s\n", filename);
+		jack_info ("getting driver descriptor from %s", filename);
 	}
 
 	if ((dlhandle = dlopen (filename, RTLD_NOW|RTLD_GLOBAL)) == NULL) {
@@ -254,7 +254,7 @@ jack_drivers_get_descriptor (JSList * drivers, const char * sofile)
 		dlsym (dlhandle, "driver_get_descriptor");
 
 	if ((dlerr = dlerror ()) != NULL) {
-		fprintf(stderr, "%s\n", dlerr);
+		jack_error("%s", dlerr);
 		dlclose (dlhandle);
 		free (filename);
 		return NULL;
@@ -371,12 +371,13 @@ static void usage (FILE *file)
 "             [ --debug-timer OR -D ]\n"
 "             [ --verbose OR -v ]\n"
 "             [ --clocksource OR -c [ c(ycle) | h(pet) | s(ystem) ]\n"
+"             [ --replace-registry OR -r ]\n"
 "             [ --silent OR -s ]\n"
 "             [ --version OR -V ]\n"
 "             [ --nozombies OR -Z ]\n"
 "         -d backend [ ... backend args ... ]\n"
 "             The backend can be `alsa', `coreaudio', `dummy',\n"
-"                                `freebob', `oss' or `portaudio'.\n\n"
+"                                `freebob', `oss', `sun', or `portaudio'.\n\n"
 "       jackd -d backend --help\n"
 "             to display options for each backend\n\n");
 }	
@@ -479,24 +480,23 @@ maybe_use_capabilities ()
   	        char c = 1;
 
 	        if (write (PIPE_WRITE_FD, &c, 1) != 1) {
-		        fprintf (stderr, "cannot write to jackstart sync "
-				 "pipe %d (%s)\n", PIPE_WRITE_FD,
+		        jack_error ("cannot write to jackstart sync "
+				 "pipe %d (%s)", PIPE_WRITE_FD,
 				 strerror (errno));
 	        }
 
 		if (close(PIPE_WRITE_FD) != 0) {
-			fprintf(stderr,
-				"jackd: error on startup pipe close: %s\n",
-				strerror (errno));
+			jack_error("jackd: error on startup pipe close: %s",
+				   strerror (errno));
 		} else {
 			/* wait for jackstart process to set our capabilities */
 			if (wait (&status) == -1) {
-				fprintf (stderr, "jackd: wait for startup "
-					 "process exit failed\n");
+				jack_error ("jackd: wait for startup "
+					    "process exit failed");
 			}
 			if (!WIFEXITED (status) || WEXITSTATUS (status)) {
-				fprintf(stderr, "jackd: jackstart did not "
-					"exit cleanly\n");
+				jack_error ("jackd: jackstart did not "
+					    "exit cleanly");
 				exit (1);
 			}
 		}
@@ -509,7 +509,7 @@ main (int argc, char *argv[])
 
 {
 	jack_driver_desc_t * desc;
-	const char *options = "-ad:P:uvshVRTFlt:mn:p:c:";
+	const char *options = "-ad:P:uvshVRZTFlt:mn:p:c:";
 	struct option long_options[] = 
 	{ 
 		{ "driver", 1, 0, 'd' },
@@ -521,6 +521,7 @@ main (int argc, char *argv[])
 		{ "name", 1, 0, 'n' },
 		{ "unlock", 0, 0, 'u' },
 		{ "realtime", 0, 0, 'R' },
+		{ "replace-registry", 0, 0, 'r' },
 		{ "realtime-priority", 1, 0, 'P' },
 		{ "timeout", 1, 0, 't' },
 		{ "temporary", 0, 0, 'T' },
@@ -538,6 +539,7 @@ main (int argc, char *argv[])
 	JSList * driver_params;
 	int driver_nargs = 1;
 	int show_version = 0;
+	int replace_registry = 0;
 	int i;
 	int rc;
 
@@ -594,6 +596,10 @@ main (int argc, char *argv[])
 			realtime_priority = atoi (optarg);
 			break;
 
+		case 'r':
+			replace_registry = 1;
+			break;
+
 		case 'R':
 			realtime = 1;
 			break;
@@ -627,8 +633,8 @@ main (int argc, char *argv[])
 			break;
 
 		default:
-			fprintf (stderr, "Unknown option character %c\n",
-				 optopt);
+			jack_error ("Unknown option character %c",
+				    optopt);
 			/*fallthru*/
 		case 'h':
 			usage (stdout);
@@ -696,7 +702,7 @@ main (int argc, char *argv[])
 
 	copyright (stdout);
 
-	rc = jack_register_server (server_name);
+	rc = jack_register_server (server_name, replace_registry);
 	switch (rc) {
 	case EEXIST:
 		fprintf (stderr, "`%s' server already active\n", server_name);
