@@ -64,6 +64,9 @@
 static pthread_mutex_t client_lock;
 static pthread_cond_t  client_ready;
 
+static int
+jack_client_close_aux (jack_client_t *client);
+
 #define EVENT_POLL_INDEX 0
 #define WAIT_POLL_INDEX 1
 #define event_fd pollfd[EVENT_POLL_INDEX].fd
@@ -86,12 +89,11 @@ init_cpu ()
 {
 	cpu_type = ((have_3dnow() << 8) | have_sse());
 	if (ARCH_X86_HAVE_3DNOW(cpu_type))
-		fprintf(stderr, "Enhanced3DNow! detected\n");
+		jack_info("Enhanced3DNow! detected");
 	if (ARCH_X86_HAVE_SSE2(cpu_type))
-		fprintf(stderr, "SSE2 detected\n");
+		jack_info("SSE2 detected");
 	if ((!ARCH_X86_HAVE_3DNOW(cpu_type)) && (!ARCH_X86_HAVE_SSE2(cpu_type)))
-		fprintf(stderr,
-			"No supported SIMD instruction sets detected\n");
+		jack_info("No supported SIMD instruction sets detected");
 	jack_port_set_funcs();
 }
 
@@ -197,7 +199,19 @@ jack_error (const char *fmt, ...)
 void 
 default_jack_error_callback (const char *desc)
 {
+#ifdef DEBUG_ENABLED
+	DEBUG("%s", desc);
+#else
 	fprintf(stderr, "%s\n", desc);
+	fflush(stderr);
+#endif
+}
+
+void 
+default_jack_info_callback (const char *desc)
+{
+	fprintf(stdout, "%s\n", desc);
+	fflush(stdout);
 }
 
 void 
@@ -206,6 +220,19 @@ silent_jack_error_callback (const char *desc)
 }
 
 void (*jack_error_callback)(const char *desc) = &default_jack_error_callback;
+void (*jack_info_callback)(const char *desc) = &default_jack_info_callback;
+
+void 
+jack_info (const char *fmt, ...)
+{
+	va_list ap;
+	char buffer[300];
+
+	va_start (ap, fmt);
+	vsnprintf (buffer, sizeof(buffer), fmt, ap);
+	jack_info_callback (buffer);
+	va_end (ap);
+}
 
 static int
 oop_client_deliver_request (void *ptr, jack_request_t *req)
@@ -246,7 +273,7 @@ jack_client_deliver_request (const jack_client_t *client, jack_request_t *req)
 	 * the server.
 	 */
 
-	return client->control->deliver_request (client->control->deliver_arg,
+	return client->deliver_request (client->deliver_arg,
 						 req);
 }
 
@@ -275,6 +302,11 @@ jack_client_alloc ()
 	client->on_shutdown = NULL;
 	client->n_port_types = 0;
 	client->port_segment = NULL;
+
+#ifdef USE_DYNSIMD
+	init_cpu();
+#endif /* USE_DYNSIMD */
+
 	return client;
 }
 
@@ -427,10 +459,10 @@ jack_client_handle_port_connection (jack_client_t *client, jack_event_t *event)
 		}
 	}
 
-	if (client->control->port_connect) {
-		client->control->port_connect (event->x.self_id, event->y.other_id,
+	if (client->control->port_connect_cbset) {
+		client->port_connect (event->x.self_id, event->y.other_id,
 					       (event->type == PortConnected ? 1 : 0), 
-					       client->control->port_connect_arg);
+					       client->port_connect_arg);
 	}
 
 	return 0;
@@ -447,8 +479,8 @@ jack_handle_reorder (jack_client_t *client, jack_event_t *event)
 	   execute it now.
 	*/
 
-	if (client->control->graph_order) {
-		client->control->graph_order (client->control->graph_order_arg);
+	if (client->control->graph_order_cbset) {
+		client->graph_order (client->graph_order_arg);
 	}
 
 	return 0;
@@ -503,8 +535,8 @@ jack_handle_reorder (jack_client_t *client, jack_event_t *event)
 	   execute it now.
 	*/
 
-	if (client->control->graph_order) {
-		client->control->graph_order (client->control->graph_order_arg);
+	if (client->control->graph_order_cbset) {
+		client->graph_order (client->graph_order_arg);
 	}
 
 	return 0;
@@ -528,7 +560,7 @@ server_connect (const char *server_name)
 	}
 
 	//JOQ: temporary debug message
-	//fprintf (stderr, "DEBUG: connecting to `%s' server\n", server_name);
+	//jack_info ("DEBUG: connecting to `%s' server", server_name);
 
 	addr.sun_family = AF_UNIX;
 	snprintf (addr.sun_path, sizeof (addr.sun_path) - 1, "%s/jack_%d",
@@ -915,12 +947,11 @@ jack_attach_port_segment (jack_client_t *client, jack_port_type_id_t ptid)
 }
 
 jack_client_t *
-jack_client_open (const char *client_name,
+jack_client_open_aux (const char *client_name,
 		  jack_options_t options,
-		  jack_status_t *status, ...)
+		  jack_status_t *status, va_list ap)
 {
 	/* optional arguments: */
-	va_list ap;			/* variable argument pointer */
 	jack_varargs_t va;		/* variable arguments */
 
 	int req_fd = -1;
@@ -930,6 +961,8 @@ jack_client_open (const char *client_name,
 	jack_port_type_id_t ptid;
 	jack_status_t my_status;
 
+	jack_messagebuffer_init ();
+	
 	if (status == NULL)		/* no status from caller? */
 		status = &my_status;	/* use local status word */
 	*status = 0;
@@ -941,9 +974,10 @@ jack_client_open (const char *client_name,
 	}
 
 	/* parse variable arguments */
-	va_start (ap, status);
-	jack_varargs_parse (options, ap, &va);
-	va_end (ap);
+	if (ap)
+        jack_varargs_parse(options, ap, &va);
+    else
+        jack_varargs_init(&va);
 
 	/* External clients need to know where the tmpdir used for
 	   communication with the server lives
@@ -984,7 +1018,7 @@ jack_client_open (const char *client_name,
 	}
 
 	/* attach the engine control/info block */
-	client->engine_shm = res.engine_shm;
+	client->engine_shm.index = res.engine_shm_index;
 	if (jack_attach_shm (&client->engine_shm)) {
 		jack_error ("cannot attached engine control shared memory"
 			    " segment");
@@ -997,7 +1031,7 @@ jack_client_open (const char *client_name,
 	jack_set_clock_source (client->engine->clock_source);
 
 	/* now attach the client control block */
-	client->control_shm = res.client_shm;
+	client->control_shm.index = res.client_shm_index;
 	if (jack_attach_shm (&client->control_shm)) {
 		jack_error ("cannot attached client control shared memory"
 			    " segment");
@@ -1027,8 +1061,8 @@ jack_client_open (const char *client_name,
 	/* set up the client so that it does the right thing for an
 	 * external client 
 	 */
-	client->control->deliver_request = oop_client_deliver_request;
-	client->control->deliver_arg = client;
+	client->deliver_request = oop_client_deliver_request;
+	client->deliver_arg = client;
 
 	if ((ev_fd = server_event_connect (client, va.server_name)) < 0) {
 		goto fail;
@@ -1073,14 +1107,22 @@ jack_client_open (const char *client_name,
 	return NULL;
 }
 
+jack_client_t* jack_client_open(const char* ext_client_name, jack_options_t options, jack_status_t* status, ...)
+{
+	va_list ap;
+	va_start(ap, status);
+	jack_client_t* res = jack_client_open_aux(ext_client_name, options, status, ap);
+	va_end(ap);
+	return res;
+}
+
 jack_client_t *
 jack_client_new (const char *client_name)
 {
 	jack_options_t options = JackUseExactName;
 	if (getenv("JACK_START_SERVER") == NULL)
 		options |= JackNoStartServer;
-
-	return jack_client_open (client_name, options, NULL);
+	return jack_client_open_aux (client_name, options, NULL, NULL);
 }
 
 char *
@@ -1170,7 +1212,7 @@ jack_internal_client_close (const char *client_name)
 		jack_error ("cannot deliver ClientUnload request to JACK "
 			    "server.");
 	}
-	
+
 	/* no response to this request */
 	
 	close (fd);
@@ -1218,8 +1260,8 @@ jack_start_freewheel (jack_client_t* client)
 #endif
 	}
 
-	if (control->freewheel_cb) {
-		control->freewheel_cb (1, control->freewheel_arg);
+	if (control->freewheel_cb_cbset) {
+		client->freewheel_cb (1, client->freewheel_arg);
 	}
 }
 
@@ -1238,8 +1280,8 @@ jack_stop_freewheel (jack_client_t* client)
 #endif
 	}
 
-	if (control->freewheel_cb) {
-		control->freewheel_cb (0, control->freewheel_arg);
+	if (control->freewheel_cb_cbset) {
+		client->freewheel_cb (0, client->freewheel_arg);
 	}
 }
 
@@ -1251,7 +1293,7 @@ jack_client_thread_suicide (jack_client_t* client)
 		client->on_shutdown (client->on_shutdown_arg);
 	} else {
 		jack_error ("jack_client_thread zombified - exiting from JACK");
-		jack_client_close (client);
+		jack_client_close_aux (client);
 		/* Need a fix : possibly make client crash if
 		 * zombified without shutdown handler 
 		 */
@@ -1270,6 +1312,8 @@ jack_client_process_events (jack_client_t* client)
 	JSList *node;
 	jack_port_t* port;
 
+	DEBUG ("process events");
+
 	if (client->pollfd[EVENT_POLL_INDEX].revents & POLLIN) {
 		
 		DEBUG ("client receives an event, "
@@ -1295,34 +1339,34 @@ jack_client_process_events (jack_client_t* client)
 					port->type_info = &client->engine->port_types[port->shared->ptype_id];
 				}
 			}
-			if (control->port_register) {
-				control->port_register
+			if (control->port_register_cbset) {
+				client->port_register
 					(event.x.port_id, TRUE,
-					 control->port_register_arg);
+					 client->port_register_arg);
 			} 
 			break;
 			
 		case PortUnregistered:
-			if (control->port_register) {
-				control->port_register
+			if (control->port_register_cbset) {
+				client->port_register
 					(event.x.port_id, FALSE,
-					 control->port_register_arg);
+					 client->port_register_arg);
 			}
 			break;
 			
 		case ClientRegistered:
-			if (control->client_register) {
-				control->client_register
+			if (control->client_register_cbset) {
+				client->client_register
 					(event.x.name, TRUE,
-					 control->client_register_arg);
+					 client->client_register_arg);
 			} 
 			break;
 			
 		case ClientUnregistered:
-			if (control->client_register) {
-				control->client_register
+			if (control->client_register_cbset) {
+				client->client_register
 					(event.x.name, FALSE,
-					 control->client_register_arg);
+					 client->client_register_arg);
 			}
 			break;
 			
@@ -1338,25 +1382,25 @@ jack_client_process_events (jack_client_t* client)
 			
 		case BufferSizeChange:
 			jack_client_invalidate_port_buffers (client);
-			if (control->bufsize) {
-				status = control->bufsize
+			if (control->bufsize_cbset) {
+				status = client->bufsize
 					(control->nframes,
-					 control->bufsize_arg);
+					 client->bufsize_arg);
 			} 
 			break;
 			
 		case SampleRateChange:
-			if (control->srate) {
-				status = control->srate
+			if (control->srate_cbset) {
+				status = client->srate
 					(control->nframes,
-					 control->srate_arg);
+					 client->srate_arg);
 			}
 			break;
 			
 		case XRun:
-			if (control->xrun) {
-				status = control->xrun
-					(control->xrun_arg);
+			if (control->xrun_cbset) {
+				status = client->xrun
+					(client->xrun_arg);
 			}
 			break;
 			
@@ -1469,7 +1513,7 @@ jack_client_core_wait (jack_client_t* client)
 		}
 	}
 
-	if (client->control->dead || client->pollfd[EVENT_POLL_INDEX].revents & ~POLLIN) {
+	if (control->dead || client->pollfd[EVENT_POLL_INDEX].revents & ~POLLIN) {
 		DEBUG ("client appears dead or event pollfd has error status\n");
 		return -1;
 	}
@@ -1480,6 +1524,8 @@ jack_client_core_wait (jack_client_t* client)
 static int
 jack_wake_next_client (jack_client_t* client)
 {
+	struct pollfd pfds[1];
+	int pret = 0;
 	char c = 0;
 
 	if (write (client->graph_next_fd, &c, sizeof (c))
@@ -1491,18 +1537,34 @@ jack_wake_next_client (jack_client_t* client)
 		return -1;
 	}
 	
-	DEBUG ("client sent message to next stage by %" PRIu64
-	       ", client reading on graph_wait_fd==%d", 
-	       jack_get_microseconds(), client->graph_wait_fd);
+	DEBUG ("client sent message to next stage by %" PRIu64 "",
+	       jack_get_microseconds());
 	
 	DEBUG("reading cleanup byte from pipe %d\n", client->graph_wait_fd);
-	
-	if ((read (client->graph_wait_fd, &c, sizeof (c))
-	     != sizeof (c))) {
-		jack_error ("cannot complete execution of the "
-			    "processing graph (%s)",
-			    strerror(errno));
-		return -1;
+
+	/* "upstream client went away?  readability is checked in
+	 * jack_client_core_wait(), but that's almost a whole cycle
+	 * before we get here.
+	 */
+
+	if (client->graph_wait_fd >= 0) {
+		pfds[0].fd = client->graph_wait_fd;
+		pfds[0].events = POLLIN;
+
+		/* 0 timeout, don't actually wait */
+		pret = poll(pfds, 1, 0);
+	}
+
+	if (pret > 0 && (pfds[0].revents & POLLIN)) {
+		if (read (client->graph_wait_fd, &c, sizeof (c))
+		    != sizeof (c)) {
+			jack_error ("cannot complete execution of the "
+				"processing graph (%s)", strerror(errno));
+			return -1;
+		}
+	} else {
+		DEBUG("cleanup byte from pipe %d not available?\n",
+			client->graph_wait_fd);
 	}
 	
 	return 0;
@@ -1526,7 +1588,7 @@ jack_thread_wait (jack_client_t* client, int status)
 
 	/* housekeeping/cleanup after data processing */
 
-	if (status == 0 && client->control->timebase_cb) {
+	if (status == 0 && client->control->timebase_cb_cbset) {
 		jack_call_timebase_master (client);
 	}
 	
@@ -1564,14 +1626,71 @@ jack_thread_wait (jack_client_t* client, int status)
 	/* begin preemption checking */
 	CHECK_PREEMPTION (client->engine, TRUE);
 	
-	if (client->control->sync_cb)
+	if (client->control->sync_cb_cbset)
 		jack_call_sync_client (client);
 
 	return client->control->nframes;
 }
 
-static void *
-jack_client_thread (void *arg)
+jack_nframes_t jack_cycle_wait (jack_client_t* client)
+{
+	/* SECTION TWO: WAIT FOR NEXT DATA PROCESSING TIME */
+
+	if (jack_client_core_wait (client)) {
+		return 0;
+	}
+
+   /* SECTION THREE: START NEXT DATA PROCESSING TIME */
+
+	/* Time to do data processing */
+
+	client->control->state = Running;
+	
+	/* begin preemption checking */
+	CHECK_PREEMPTION (client->engine, TRUE);
+	
+	if (client->control->sync_cb_cbset)
+		jack_call_sync_client (client);
+
+	return client->control->nframes;
+}
+
+void jack_cycle_signal(jack_client_t* client, int status)
+{
+	client->control->last_status = status;
+
+   /* SECTION ONE: HOUSEKEEPING/CLEANUP FROM LAST DATA PROCESSING */
+
+	/* housekeeping/cleanup after data processing */
+
+	if (status == 0 && client->control->timebase_cb_cbset) {
+		jack_call_timebase_master (client);
+	}
+	
+	/* end preemption checking */
+	CHECK_PREEMPTION (client->engine, FALSE);
+	
+	client->control->finished_at = jack_get_microseconds();
+	
+	/* wake the next client in the chain (could be the server), 
+	   and check if we were killed during the process
+	   cycle.
+	*/
+	
+	if (jack_wake_next_client (client)) {
+		DEBUG("client cannot wake next, or is dead\n");
+		jack_client_thread_suicide (client);
+		/*NOTREACHED*/
+	}
+
+	if (status || client->control->dead || !client->engine->engine_ok) {
+		jack_client_thread_suicide (client);
+		/*NOTREACHED*/
+	}
+}
+
+static void
+jack_client_thread_aux (void *arg)
 {
 	jack_client_t *client = (jack_client_t *) arg;
 	jack_client_control_t *control = client->control;
@@ -1587,22 +1706,34 @@ jack_client_thread (void *arg)
 
 	DEBUG ("client thread is now running");
 
-	if (control->thread_init) {
+	if (control->thread_init_cbset) {
 		DEBUG ("calling client thread init callback");
-		control->thread_init (control->thread_init_arg);
+		client->thread_init (client->thread_init_arg);
 	}
 
 	/* wait for first wakeup from server */
 
 	if (jack_thread_first_wait (client) == control->nframes) {
+
 		/* now run till we're done */
-		if (control->process) {
+
+		if (control->process_cbset) {
+
 			/* run process callback, then wait... ad-infinitum */
-			while (jack_thread_wait (client, 
-						 control->process (control->nframes, 
-								   control->process_arg)) ==
-			       control->nframes)
-				;
+
+			while (1) {
+				DEBUG("client calls process()");
+				int status = (client->process (control->nframes, 
+								client->process_arg) ==
+					      control->nframes);
+				control->state = Finished;
+				DEBUG("client leaves process(), re-enters wait");
+				if (!jack_thread_wait (client, status)) {
+					break;
+				}
+				DEBUG("client done with wait");
+			}
+
 		} else {
 			/* no process handling but still need to process events */
 			while (jack_thread_wait (client, 0) == control->nframes)
@@ -1611,6 +1742,31 @@ jack_client_thread (void *arg)
 	}
 
 	jack_client_thread_suicide (client);
+}
+
+static void *
+jack_client_thread (void *arg)
+{
+	jack_client_t *client = (jack_client_t *) arg;
+	jack_client_control_t *control = client->control;
+	
+	if (client->control->thread_cb_cbset) {
+	
+		pthread_mutex_lock (&client_lock);
+		client->thread_ok = TRUE;
+		client->thread_id = pthread_self();
+		pthread_cond_signal (&client_ready);
+		pthread_mutex_unlock (&client_lock);
+
+		control->pid = getpid();
+		control->pgrp = getpgrp();
+
+		client->thread_cb(client->thread_cb_arg);
+		jack_client_thread_suicide(client);
+	} else {
+		jack_client_thread_aux(arg);
+	}
+	
 	/*NOTREACHED*/
 	return (void *) 0;
 }
@@ -1626,10 +1782,10 @@ jack_client_process_thread (void *arg)
 	jack_client_control_t *control = client->control;
 	int err = 0;
       
-	if (client->control->thread_init) {
+	if (client->control->thread_init_cbset) {
 	/* this means that the init callback will be called twice -taybin*/
 		DEBUG ("calling client thread init callback");
-		client->control->thread_init (client->control->thread_init_arg);
+		client->thread_init (client->thread_init_arg);
 	}
 
 	client->control->pid = getpid();
@@ -1650,19 +1806,19 @@ jack_client_process_thread (void *arg)
 		 
 		control->state = Running;
 
-		if (control->sync_cb)
+		if (control->sync_cb_cbset)
 			jack_call_sync_client (client);
 
-		if (control->process) {
-			if (control->process (control->nframes,
-				control->process_arg) == 0) {
-					control->state = Finished;
+		if (control->process_cbset) {
+			if (client->process (control->nframes,
+					     client->process_arg) == 0) {
+				control->state = Finished;
 			}
 		} else {
 			control->state = Finished;
 		}
 
-		if (control->timebase_cb)
+		if (control->timebase_cb_cbset)
 			jack_call_timebase_master (client);
                 
 		control->finished_at = jack_get_microseconds();
@@ -1699,7 +1855,7 @@ jack_client_process_thread (void *arg)
 		jack_error ("jack_client_process_thread zombified - exiting from JACK");
 		/* Need a fix : possibly make client crash if
 		 * zombified without shutdown handler */
-		jack_client_close (client); 
+		jack_client_close_aux (client); 
 	}
 
 	pthread_exit (0);
@@ -1865,8 +2021,8 @@ jack_activate (jack_client_t *client)
 	return jack_client_deliver_request (client, &req);
 }
 
-int 
-jack_deactivate (jack_client_t *client)
+static int 
+jack_deactivate_aux (jack_client_t *client)
 {
 	jack_request_t req;
 	int rc = ESRCH;			/* already shut down */
@@ -1882,14 +2038,20 @@ jack_deactivate (jack_client_t *client)
 	return rc;
 }
 
-int
-jack_client_close (jack_client_t *client)
+int 
+jack_deactivate (jack_client_t *client)
+{
+	return jack_deactivate_aux(client);
+}
+
+static int
+jack_client_close_aux (jack_client_t *client)
 {
 	JSList *node;
 	void *status;
 	int rc;
 
-	rc = jack_deactivate (client);
+	rc = jack_deactivate_aux (client);
 	if (rc == ESRCH) {		/* already shut down? */
 		return rc;
 	}
@@ -1934,11 +2096,11 @@ jack_client_close (jack_client_t *client)
 		}
 
 #ifndef JACK_USE_MACH_THREADS
-		if (client->graph_wait_fd) {
+		if (client->graph_wait_fd >= 0) {
 			close (client->graph_wait_fd);
 		}
 		
-		if (client->graph_next_fd) {
+		if (client->graph_next_fd >= 0) {
 			close (client->graph_next_fd);
 		}
 #endif		
@@ -1962,8 +2124,15 @@ jack_client_close (jack_client_t *client)
 	}
 	jack_slist_free (client->ports_ext);
 	jack_client_free (client);
+	jack_messagebuffer_exit ();
 
 	return rc;
+}
+
+int
+jack_client_close (jack_client_t *client)
+{
+	return jack_client_close_aux(client);
 }	
 
 int 
@@ -2054,6 +2223,12 @@ jack_set_error_function (void (*func) (const char *))
 	jack_error_callback = func;
 }
 
+void
+jack_set_info_function (void (*func) (const char *))
+{
+	jack_info_callback = func;
+}
+
 int 
 jack_set_graph_order_callback (jack_client_t *client,
 			       JackGraphOrderCallback callback, void *arg)
@@ -2062,8 +2237,9 @@ jack_set_graph_order_callback (jack_client_t *client,
 		jack_error ("You cannot set callbacks on an active client.");
 		return -1;
 	}
-	client->control->graph_order = callback;
-	client->control->graph_order_arg = arg;
+	client->graph_order = callback;
+	client->graph_order_arg = arg;
+	client->control->graph_order_cbset = (callback != NULL);
 	return 0;
 }
 
@@ -2075,8 +2251,9 @@ int jack_set_xrun_callback (jack_client_t *client,
 		return -1;
 	}
 
-	client->control->xrun = callback;
-	client->control->xrun_arg = arg;
+	client->xrun = callback;
+	client->xrun_arg = arg;
+	client->control->xrun_cbset = (callback != NULL);
 	return 0;       
 }
 
@@ -2089,8 +2266,15 @@ jack_set_process_callback (jack_client_t *client,
 		jack_error ("You cannot set callbacks on an active client.");
 		return -1;
 	}
-	client->control->process_arg = arg;
-	client->control->process = callback;
+	
+	if (client->control->thread_cb_cbset) {
+		jack_error ("A thread callback has already been setup, both models cannot be used at the same time!");
+		return -1;
+	}
+	
+	client->process_arg = arg;
+	client->process = callback;
+	client->control->process_cbset = (callback != NULL);
 	return 0;
 }
 
@@ -2103,8 +2287,9 @@ jack_set_thread_init_callback (jack_client_t *client,
 		jack_error ("You cannot set callbacks on an active client.");
 		return -1;
 	}
-	client->control->thread_init_arg = arg;
-	client->control->thread_init = callback;
+	client->thread_init_arg = arg;
+	client->thread_init = callback;
+	client->control->thread_init_cbset = (callback != NULL);
 	return 0;
 }
 
@@ -2116,8 +2301,9 @@ jack_set_freewheel_callback (jack_client_t *client,
 		jack_error ("You cannot set callbacks on an active client.");
 		return -1;
 	}
-	client->control->freewheel_arg = arg;
-	client->control->freewheel_cb = callback;
+	client->freewheel_arg = arg;
+	client->freewheel_cb = callback;
+	client->control->freewheel_cb_cbset = (callback != NULL);
 	return 0;
 }
 
@@ -2125,8 +2311,9 @@ int
 jack_set_buffer_size_callback (jack_client_t *client,
 			       JackBufferSizeCallback callback, void *arg)
 {
-	client->control->bufsize_arg = arg;
-	client->control->bufsize = callback;
+	client->bufsize_arg = arg;
+	client->bufsize = callback;
+	client->control->bufsize_cbset = (callback != NULL);
 	return 0;
 }
 
@@ -2139,8 +2326,9 @@ jack_set_port_registration_callback(jack_client_t *client,
 		jack_error ("You cannot set callbacks on an active client.");
 		return -1;
 	}
-	client->control->port_register_arg = arg;
-	client->control->port_register = callback;
+	client->port_register_arg = arg;
+	client->port_register = callback;
+	client->control->port_register_cbset = (callback != NULL);
 	return 0;
 }
 
@@ -2153,8 +2341,9 @@ jack_set_port_connect_callback(jack_client_t *client,
 		jack_error ("You cannot set callbacks on an active client.");
 		return -1;
 	}
-	client->control->port_connect_arg = arg;
-	client->control->port_connect = callback;
+	client->port_connect_arg = arg;
+	client->port_connect = callback;
+	client->control->port_connect_cbset = (callback != NULL);
 	return 0;
 }
 
@@ -2167,8 +2356,28 @@ jack_set_client_registration_callback(jack_client_t *client,
 		jack_error ("You cannot set callbacks on an active client.");
 		return -1;
 	}
-	client->control->client_register_arg = arg;
-	client->control->client_register = callback;
+	client->client_register_arg = arg;
+	client->client_register = callback;
+	client->control->client_register_cbset = (callback != NULL);
+	return 0;
+}
+
+int
+jack_set_process_thread(jack_client_t* client, JackThreadCallback callback, void *arg)
+{
+	if (client->control->active) {
+		jack_error ("You cannot set callbacks on an active client.");
+		return -1;
+	}
+	
+	if (client->control->process_cbset) {
+		jack_error ("A process callback has already been setup, both models cannot be used at the same time!");
+		return -1;
+	}
+
+	client->thread_cb_arg = arg;
+	client->thread_cb = callback;
+	client->control->thread_cb_cbset = (callback != NULL);
 	return 0;
 }
 
