@@ -100,7 +100,6 @@ static char jack_shm_server_prefix[JACK_SERVER_NAME_SIZE] = "";
  * again on that machine until after a reboot.
  */
 
-#define JACK_SEMAPHORE_KEY 0x282929
 #ifndef USE_POSIX_SHM
 #define JACK_SHM_REGISTRY_KEY JACK_SEMAPHORE_KEY
 #endif
@@ -243,7 +242,7 @@ jack_set_server_prefix (const char *server_name)
  * returns: 0 if successful
  */
 static int
-jack_server_initialize_shm (void)
+jack_server_initialize_shm (int new_registry)
 {
 	int rc;
 
@@ -254,6 +253,11 @@ jack_server_initialize_shm (void)
 
 	rc = jack_access_registry (&registry_info);
 
+	if (new_registry) {
+		jack_remove_shm (&registry_id);
+		rc = ENOENT;
+	}
+
 	switch (rc) {
 	case ENOENT:			/* registry does not exist */
 		rc = jack_create_registry (&registry_info);
@@ -302,6 +306,7 @@ jack_initialize_shm (const char *server_name)
 	jack_set_server_prefix (server_name);
 
 	jack_shm_lock_registry ();
+
 	if ((rc = jack_access_registry (&registry_info)) == 0) {
 		if ((rc = jack_shm_validate_registry ()) != 0) {
 			jack_error ("Incompatible shm registry, "
@@ -373,16 +378,16 @@ jack_release_shm_info (jack_shm_registry_index_t index)
  *	   ENOMEM if unable to access shared memory registry
  */
 int
-jack_register_server (const char *server_name)
+jack_register_server (const char *server_name, int new_registry)
 {
 	int i;
 	pid_t my_pid = getpid ();
 
 	jack_set_server_prefix (server_name);
 
-	fprintf (stderr, "JACK compiled with %s SHM support.\n", JACK_SHM_TYPE);
+	jack_info ("JACK compiled with %s SHM support.", JACK_SHM_TYPE);
 
-	if (jack_server_initialize_shm ())
+	if (jack_server_initialize_shm (new_registry))
 		return ENOMEM;
 
 	jack_shm_lock_registry ();
@@ -647,6 +652,11 @@ static void
 jack_remove_shm (jack_shm_id_t *id)
 {
 	/* registry may or may not be locked */
+	/* note that in many cases the client has already removed
+	   the shm segment, so this failing is not an error.
+	   XXX it would be good to differentiate between these
+	   two conditions.
+	*/
 	shm_unlink ((char *) id);
 }
 
@@ -838,6 +848,11 @@ static void
 jack_remove_shm (jack_shm_id_t *id)
 {
 	/* registry may or may not be locked */
+	/* this call can fail if we are attempting to 
+	   remove a segment that was already deleted
+	   by the client. XXX i suppose the 
+	   function should take a "canfail" argument.
+	*/
 	shmctl (*id, IPC_RMID, NULL);
 }
 
