@@ -152,7 +152,11 @@ jack_client_create_thread (jack_client_t* client,
 		log_result("requesting joinable thread creation", result);
 		return result;
 	}
+#ifdef __OpenBSD__
+	result = pthread_attr_setscope(&attr, PTHREAD_SCOPE_PROCESS);
+#else
 	result = pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);
+#endif
 	if (result) {
 		log_result("requesting system scheduling scope", result);
 		return result;
@@ -194,6 +198,26 @@ jack_client_create_thread (jack_client_t* client,
 	return 0;
 }
 
+int
+jack_client_real_time_priority (jack_client_t* client)
+{
+	if (!client->engine->real_time) {
+		return -1;
+	}
+	
+	return client->engine->client_priority;
+}
+
+int
+jack_client_max_real_time_priority (jack_client_t* client)
+{
+	if (!client->engine->real_time) {
+		return -1;
+	}
+
+	return client->engine->max_client_priority;
+}
+
 #if JACK_USE_MACH_THREADS 
 
 int
@@ -252,3 +276,4 @@ jack_acquire_real_time_scheduling (pthread_t thread, int priority)
 }
 
 #endif /* JACK_USE_MACH_THREADS */
+
